@@ -1,5 +1,7 @@
 /***************************************************************************
-* Copyright (c) 2016, Johan Mabille and Sylvain Corlay                     *
+* Copyright (c) Johan Mabille, Sylvain Corlay, Wolf Vollprecht and         *
+* Martin Renou                                                             *
+* Copyright (c) QuantStack                                                 *
 *                                                                          *
 * Distributed under the terms of the BSD 3-Clause License.                 *
 *                                                                          *
@@ -50,19 +52,35 @@
 #ifdef XSIMD_FORCE_X86_INSTR_SET
     #define XSIMD_X86_INSTR_SET XSIMD_FORCE_X86_INSTR_SET
     #define XSIMD_X86_INSTR_SET_AVAILABLE XSIMD_VERSION_NUMBER_AVAILABLE
-    #warning "Forcing X86 instruction set"
+    #ifdef _MSC_VER
+        #pragma message("Warning: Forcing X86 instruction set")
+    #else
+        #warning "Forcing X86 instruction set"
+    #endif
 #elif defined(XSIMD_FORCE_X86_AMD_INSTR_SET)
     #define XSIMD_X86_AMD_INSTR_SET XSIMD_FORCE_X86_AMD_INSTR_SET
     #define XSIMD_X86_AMD_INSTR_SET_AVAILABLE XSIMD_VERSION_NUMBER_AVAILABLE
-    #warning "Forcing X86 AMD instruction set"
+    #ifdef _MSC_VER
+        #pragma message("Warning: Forcing X86 AMD instruction set")
+    #else
+        #warning "Forcing X86 AMD instruction set"
+    #endif
 #elif defined(XSIMD_FORCE_PPC_INSTR_SET)
     #define XSIMD_PPC_INSTR_SET XSIMD_FORCE_PPC_INSTR_SET
     #define XSIMD_PPC_INSTR_SET_AVAILABLE XSIMD_VERSION_NUMBER_AVAILABLE
-    #warning "Forcing PPC instruction set"
+    #ifdef _MSC_VER
+        #pragma message("Warning: Forcing PPC instruction set")
+    #else
+        #warning "Forcing PPC instruction set"
+    #endif
 #elif defined(XSIMD_FORCE_ARM_INSTR_SET)
     #define XSIMD_ARM_INSTR_SET XSIMD_FORCE_ARM_INSTR_SET
     #define XSIMD_ARM_INSTR_SET_AVAILABLE XSIMD_VERSION_NUMBER_AVAILABLE
-    #warning "Forcing ARM instruction set"
+    #ifdef _MSC_VER
+        #pragma message("Warning: Forcing ARM instruction set")
+    #else
+        #warning "Forcing ARM instruction set"
+    #endif
 #endif
 
 /***********************
